-- get the addon namespace
local addon, ns = ...
-- get the config values
local cfg = ns.cfg
-- holder for some lib functions
local lib = _G["HEDD_lib"] or CreateFrame("Frame","HEDD_lib")
ns.lib = lib

lib.AddRuneSpell = function(spell,ids,runemask)
	if lib.AddSpell(spell,ids,true) then
		cfg.spells[spell].rune=true
		cfg.spells[spell].noupdate=nil
		cfg.spells[spell].red = runemask[1]
		cfg.spells[spell].green = runemask[2]
		cfg.spells[spell].blue = runemask[3]
		cfg.spells[spell].numrunes = cfg.spells[spell].red + cfg.spells[spell].green + cfg.spells[spell].blue
		lib.UpdateSpell(spell)
		return true
	end
	return false
end

lib.updateRune = function(rune)
	if not rune then return nil end
	runeType = GetRuneType(rune)
	if runeType then
		if not cfg.runes then
			cfg.runes={}
		end
		if not cfg.runes[rune] then
			cfg.runes[rune]={}
			cfg.runes[rune].runeType=runeType
			cfg.runes[rune].start, cfg.runes[rune].duration, cfg.runes[rune].runeReady = GetRuneCooldown(rune)
			return true
		else
			pstart,pduration,pruneReady,pruneType = cfg.runes[rune].start, cfg.runes[rune].duration, cfg.runes[rune].runeReady,runeType
			cfg.runes[rune].runeType=runeType
			cfg.runes[rune].start, cfg.runes[rune].duration, cfg.runes[rune].runeReady = GetRuneCooldown(rune)
			if pstart~=cfg.runes[rune].start or pduration~=cfg.runes[rune].duration or pruneReady~=cfg.runes[rune].runeReady or pruneType~=cfg.runes[rune].runeType then
				return true
			end
		end
	end
	return nil
end

lib.updateAllRunes = function()
	allrunesupdate=nil
	for i_runes = 1, 6 do
		if lib.updateRune(i_runes) then allrunesupdate=true end
	end
	return allrunesupdate
end

lib.RuneTypeCD = function(runeType)
	c_rune = 0
	tl_rune=99999
	numrunes=0

	if runeType==1 then
		start,finish=1,2
	elseif runeType==2 then
		start,finish=3,4
	else
		start,finish=5,6
	end
	
	for i_runes = start, finish do
		tl_rune_any =  cfg.runes[i_runes].duration - (GetTime() - cfg.runes[i_runes].start)
		tl_rune_any = (tl_rune_any>=0) and tl_rune_any or 0
		if tl_rune_any==0 then
			numrunes=numrunes+1
		else
			if tl_rune_any<tl_rune then
				tl_rune=tl_rune_any
			end
		end
	end
	return numrunes, tl_rune
end

lib.GetRuneCD = function(rune)
	if not cfg.runes or not cfg.runes[rune] then return 0 end
	tl_rune =  cfg.runes[rune].duration - (GetTime() - cfg.runes[rune].start)
	tl_rune = (tl_rune>=0) and tl_rune or 0

	return tl_rune
end

lib.FullDeplatedRunes = function()
	allrunes=0
	for i_runes = 1,3 do
		numrunes=0
		c_rune = 0
		tl_rune=99999
		start,finish=i_runes*2-1,i_runes*2
		
		for i_runes = start, finish do
			tl_rune_any =  cfg.runes[i_runes].duration - (GetTime() - cfg.runes[i_runes].start)
			tl_rune_any = (tl_rune_any>=0) and tl_rune_any or 0
			if tl_rune_any==0 then
				numrunes=numrunes+1
			end
		end
		allrunes = (numrunes==0) and (allrunes+1) or 0
	end
	return allrunes
end

lib.getruneCD = function(runeType,useDeath)
	c_rune = 0
	tl_rune=99999
	ReadyRuneFound=nil
	
	if runeType==1 then
		start,finish=1,2
	elseif runeType==2 then
		start,finish=3,4
	else
		start,finish=5,6
	end
	
	for i_runes = start, finish do
		if ReadyRuneFound then return 0 end
		if not ExcludeRunes[i_runes] and cfg.runes[i_runes].runeType==runeType then
			tl_rune_any =  cfg.runes[i_runes].duration - (GetTime() - cfg.runes[i_runes].start)
			tl_rune_any = (tl_rune_any>=0) and tl_rune_any or 0
			if tl_rune_any==0 then
				ReadyRuneFound=true
				ExcludeRunes[i_runes]=true
			else
				if tl_rune_any<tl_rune then
					tl_rune=tl_rune_any
					c_rune=i_runes
				end
			end
		end
	end
	if ReadyRuneFound then return 0 end
	
	if useDeath then
		for i_runes = 1, 6 do
			if ReadyRuneFound then return 0 end
			if not ExcludeRunes[i_runes] and cfg.runes[i_runes].runeType==4 then
				tl_rune_any =  cfg.runes[i_runes].duration - (GetTime() - cfg.runes[i_runes].start)
				tl_rune_any = (tl_rune_any>=0) and tl_rune_any or 0
				if tl_rune_any==0 then
					ReadyRuneFound=true
					ExcludeRunes[i_runes]=true
				else
					if tl_rune_any<tl_rune then
						tl_rune=tl_rune_any
						c_rune=i_runes
					end
				end
			end
		end	
	end
	if ReadyRuneFound then return 0 end
	ExcludeRunes[c_rune]=true
	
	return tl_rune
end

lib.RuneCDCheck = function (spell,useDeath,twospells,useGCD,minWait)
	minWait=minWait or 0
	ExcludeRunes = {}

	if cfg.spells[spell] then
		if useGCD and cfg.gcd then
			tl_spell=cfg.spells[cfg.gcd].cd - (GetTime() - cfg.spells[cfg.gcd].start)
		else
			tl_spell=cfg.spells[spell].cd - (GetTime() - cfg.spells[spell].start)
		end
		tl_spell = (tl_spell>=0) and tl_spell or 0
		
		tl_rune_r,tl_rune_g,tl_rune_b = 0,0,0
		n_rune_r,n_rune_g,n_rune_b = 0,0,0
		
		for j = 1,2 do
			if cfg.spells[spell].red==1 then
				tl_rune_r=lib.getruneCD(1,useDeath)
				if ReadyRuneFound then n_rune_r = n_rune_r+1 end
			else
				n_rune_r=2
			end
		
			if cfg.spells[spell].green==1 then
				tl_rune_g=lib.getruneCD(2,useDeath)
				if ReadyRuneFound then n_rune_g = n_rune_g+1 end
			else
				n_rune_g=2
			end
		
			if cfg.spells[spell].blue==1 then
				tl_rune_b=lib.getruneCD(3,useDeath)
				if ReadyRuneFound then n_rune_b = n_rune_b+1 end
			else
				n_rune_b=2
			end
			
			if not twospells then break end
		end
		
		if twospells and n_rune_r==2 and n_rune_g==2 and n_rune_b==2 then
			lib.UpdateBar(spell,0)
			return true
		end
		
		if n_rune_r>0 and n_rune_g>0 and n_rune_b>0 then
			tl_spell=tl_spell
		else
			if tl_spell<tl_rune_r then tl_spell=tl_rune_r end
			if tl_spell<tl_rune_g then tl_spell=tl_rune_g end
			if tl_spell<tl_rune_b then tl_spell=tl_rune_b end
		end
		
		if tl_spell==0 or tl_spell<=minWait or tl_spell==cfg.timeleft then
			lib.UpdateBar(spell,tl_spell)
			cfg.ctimeleft=tl_spell
			return true
		else
			if cfg.ctimeleft>tl_spell then
				cfg.ctimeleft=tl_spell
			end
		end
	end
	return nil
end

lib.GetRuneSpellCD=function(spell,useDeath,useGCD)
	ExcludeRunes = {}
	if cfg.spells[spell] then
		if useGCD then
			tl_spell=cfg.spells[cfg.gcd].cd - (GetTime() - cfg.spells[cfg.gcd].start)
		else
			tl_spell=cfg.spells[spell].cd - (GetTime() - cfg.spells[spell].start)
		end
		tl_spell = (tl_spell>=0) and tl_spell or 0
		
		tl_rune_r,tl_rune_g,tl_rune_b = 0,0,0
		n_rune_r,n_rune_g,n_rune_b = 0,0,0

		if cfg.spells[spell].red==1 then
			tl_rune_r=lib.getruneCD(1,useDeath)
			if ReadyRuneFound then n_rune_r = n_rune_r+1 end
		else
			n_rune_r=1
		end
		
		if cfg.spells[spell].green==1 then
			tl_rune_g=lib.getruneCD(2,useDeath)
			if ReadyRuneFound then n_rune_g = n_rune_g+1 end
		else
			n_rune_g=1
		end
		
		if cfg.spells[spell].blue==1 then
			tl_rune_b=lib.getruneCD(3,useDeath)
			if ReadyRuneFound then n_rune_b = n_rune_b+1 end
		else
			n_rune_b=1
		end
			
		if n_rune_r>0 and n_rune_g>0 and n_rune_b>0 then
			tl_spell=tl_spell
		else
			if tl_spell<tl_rune_r then tl_spell=tl_rune_r end
			if tl_spell<tl_rune_g then tl_spell=tl_rune_g end
			if tl_spell<tl_rune_b then tl_spell=tl_rune_b end
		end

		return tl_spell
	end
	return 0
end
